﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Windows;
using System.Drawing;
using System.Collections.Specialized;
using System.Windows.Forms;
using System.Windows.Input;
using System.Timers;
using System.Windows.Threading;

namespace WebScreenSaver
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : System.Windows.Application
    {
        //load app.config
        private bool _interactiveModeAllowed = WebScreenSaver.Properties.Settings.Default.InteractiveModeAllowed;// whether user can press 'I' key to interact with displayed web page
        private bool _saveScreen = WebScreenSaver.Properties.Settings.Default.SaveScreen;// whether the screen should periodically fade the screen to white and black to help prevent monitor burn in.
        private StringCollection _displayPaths = WebScreenSaver.Properties.Settings.Default.DisplayPaths;

        //get screen config
        private int _numberOfMonitors = SystemInformation.MonitorCount;

        //window lists
        private List<Window> _webWindows = new List<Window>();
        private List<Window> _overlayWindows = new List<Window>();

        //timed events
        private System.Timers.Timer _showWebTimer = new System.Timers.Timer();
        private System.Timers.Timer _monitorTimer = new System.Timers.Timer();

        //application state
        private bool _isInteracting = false;
        private DateTime _lastShowedScreenSave = DateTime.Now;
        //input detection vars
        DateTime _lastEscapeKeyPress = DateTime.Now;
        DateTime _lastDetectedInput = DateTime.Now;
        bool _isMouseActive = false;
        System.Windows.Point _mousePosition;


        private void Application_Startup(object sender, StartupEventArgs e) {


            string[] args = e.Args;


            if (args.Length > 0)
            {
                // Get the 2 character command line argument
                string arg = args[0].ToLower().Trim().Substring(0, 2); //debug uncomment this
                
                switch (arg)
                {
                    case "/c":
                        // Show the options dialog
                        Settings settings = new Settings();
                        settings.Show();
                        break;
                    case "/p":
                        // Don't do anything for preview
                        System.Windows.Application.Current.Shutdown();
                        break;
                    case "/s":
                        // Show screensaver
                        StartScreenSaver();
                        break;
                    default:
                        System.Windows.Application.Current.Shutdown();
                        break;
                }
            }
            else
            {
                // If no arguments were passed in, show the screensaver
                StartScreenSaver();

            }

        }


        private string GetPath(StringCollection paths, int pathIndex) {
            if (pathIndex >= paths.Count)
            {
                return "";
            }
            else {
                return paths[pathIndex].ToString();
            }
        }

        private void StartScreenSaver() {

            // loop through screens and populate lists for each layer of windows
            for (int monitor = 0; monitor < _numberOfMonitors; monitor++)
            {
                //get monitor bounds/dimensions
                System.Drawing.Rectangle monitorArea = System.Windows.Forms.Screen.AllScreens[monitor].Bounds;
 
                
                // create website display layer and subscribe to input events
                _webWindows.Add( new Screen(GetPath(_displayPaths, monitor), monitor, monitorArea.Left, monitorArea.Top, monitorArea.Width, monitorArea.Height));
                //register for input events
                _webWindows[monitor].MouseMove += new System.Windows.Input.MouseEventHandler(User_MouseMove);
                _webWindows[monitor].MouseDown += new System.Windows.Input.MouseButtonEventHandler(User_MouseDown);
                _webWindows[monitor].KeyDown += new System.Windows.Input.KeyEventHandler(User_KeyDown);
                
                
                // create overlay fading layer and subscribe to input events
                _overlayWindows.Add(new Overlay(monitor, monitorArea.Left, monitorArea.Top, monitorArea.Width, monitorArea.Height));
                //register for input events
                _overlayWindows[monitor].MouseMove += new System.Windows.Input.MouseEventHandler(User_MouseMove);
                _overlayWindows[monitor].MouseDown += new System.Windows.Input.MouseButtonEventHandler(User_MouseDown);
                _overlayWindows[monitor].KeyDown += new System.Windows.Input.KeyEventHandler(User_KeyDown);
            
            }

            ShowOverlayWindows();

            //set timer to activate web windows under overlay windows
            _showWebTimer.Elapsed += new ElapsedEventHandler(ShowWebWindows);
            // set the interval to 1.1 seconds
            _showWebTimer.Interval = 1100;
            _showWebTimer.Enabled = true;


            //set timer to activate screen save and to monitor if user left interactive mode on.
            _monitorTimer.Elapsed += new ElapsedEventHandler(MonitorStatus);
            // set the interval to 10 seconds
            _monitorTimer.Interval = 10 * 1000;
            _monitorTimer.Enabled = true;


            //the end
        }

        private void ShowOverlayWindows() {
            // set overlay on top of web windows

            this.Dispatcher.Invoke(DispatcherPriority.Normal, (Action)(() =>
            {
                foreach (Window i in _overlayWindows)
                {
                    i.Topmost = true;
                    i.Show();
                    i.Cursor = System.Windows.Input.Cursors.None;
                }

            }));
        }

        private void HideOverlayWindows() {
            // set overlay on top of web windows

            this.Dispatcher.Invoke(DispatcherPriority.Normal, (Action)(() =>
            {
                foreach (Window i in _overlayWindows)
                {
                    i.Hide();
                    System.Windows.Forms.Cursor.Show();
                }

            }));
        }

        private void MonitorStatus(object source, ElapsedEventArgs e) {
            //check to see if user left the screensaver in interactive mode for too long
            if (_isInteracting && (_lastDetectedInput.AddMinutes(5) < DateTime.Now)) {
                ToggleInteractiveMode();
            }


            //check to see if we need to run screensaver and run it if we do
            if ( _saveScreen && ( _lastShowedScreenSave.AddMinutes(5) < DateTime.Now )) {


                this.Dispatcher.Invoke(DispatcherPriority.Normal, (Action)(() =>
                {
                        //save each screen
                        foreach (Overlay i in _overlayWindows)
                        {
                            i.SaveScreen();
                        }

                }));

                  _lastShowedScreenSave = DateTime.Now;
            }
        
        }

        private void ShowWebWindows(object source, ElapsedEventArgs e)
        {

            // update from another thread see: http://thispointer.spaces.live.com/blog/cns!74930F9313F0A720!252.entry?_c11_blogpart_blogpart=blogview&_c=blogpart&wa=wsignin1.0

            this.Dispatcher.Invoke(DispatcherPriority.Normal, (Action)(() =>
            {

                // set web windows to top and show
                foreach (Window i in _webWindows)
                {
                    //i.Topmost = true;
                    i.Show();
                }

            }));

            // disable the timer as we only need it once
            _showWebTimer.Enabled = false;

        }



    #region "Input Event Handlers"
        private void ToggleInteractiveMode() { 
                        if (_isInteracting)
                {
                    _isInteracting = false;
                    ShowOverlayWindows();
                    _isMouseActive = false;

                }
                else
                {
                    _isInteracting = true;
                    HideOverlayWindows();
                }   
        }

        void User_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {
            if (e.Key == Key.Escape && _interactiveModeAllowed)
            {
                if (DateTime.Now > _lastEscapeKeyPress.AddMilliseconds(400)) {
                    _lastEscapeKeyPress = DateTime.Now;
                    ToggleInteractiveMode();
                }
            }
            else { 
                InputDetected();
            }
        }

        void User_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            InputDetected();
        }

        void User_MouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {

            System.Windows.Point currentPosition = e.MouseDevice.GetPosition((Window)sender);
            // Set IsActive and MouseLocation only the first time this event is called.
            if (!_isMouseActive)
            {
                _mousePosition = currentPosition;
                _isMouseActive = true;
            }
            else
            {
                // If the mouse has moved significantly since first call, close.
                if ((Math.Abs(_mousePosition.X - currentPosition.X) > 60) ||
                    (Math.Abs(_mousePosition.Y - currentPosition.Y) > 60))
                {
                    InputDetected();
                }
            }
        }

        private void InputDetected()
        {
            _lastDetectedInput = DateTime.Now;

            if (!_isInteracting) { 
                System.Windows.Application.Current.Shutdown();            
            }

        }
    #endregion

        void Path_Loaded(object sender, System.Windows.Navigation.NavigationEventArgs e)
        {
            ShowOverlayWindows();
        }



    }
}
